Unit Pool_Mod_Unit;

interface

{
   Things to Add:

   - Graphic instead of buttons (TrueSpace?)
   - "Best" teams for a player (e.g., you can always pick a Redskins game...)

   - Don't create a persistent session until they really log in (Hard!)
}

{$DEFINE PAYMENT_PENALTY}        // Remove the "NO_" to turn payment penalties on
{$DEFINE NO_TIME_CHECK}          // Remove for Registered Users

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WABD_Objects, DBTables, Db, TableUtils;

const
   POOL_ALIAS     = 'WABD_POOL_DB';
   ADMIN_USER_ID  = 1;
   GUEST_USER_ID  = -1;
   MAXWEEK        = 17;    // Max Number of weeks in a season
   WIN_COLOR      = clGreen;
   LOSE_COLOR     = clRed;
   DROP_COLOR     = clRed;

type
   TLogAction = (Log_CreateTables, Log_LogOn, Log_LogOff, Log_TimeOut,
      Log_NewUser, Log_SavePicks, Log_ViewAllPicks, Log_LookStats,
      Log_LookResults, Log_TeamStats, Log_PlayerStats);

   TEachTable = procedure(Table: TTable) of object;

   TPlayerScorePair = class
      PlayerID : integer;
      Score    : double;
      Name     : string;
   end;
   
   TViewRecord = class
      UserID      : integer;
      UserName    : string;
      NumRight    : integer;
      TiePoints   : double;
   end;

   TViewSortType = (vsName, vsPicks);

   TPlayerStat = class
      UserID      : integer;
      UserName    : string;
      WeeksPlayed : integer;
      PickPer     : double;
      NumRight    : array[1..MAXWEEK] of integer;
      Dropped     : array[1..MAXWEEK] of boolean;
      Played      : array[1..MAXWEEK] of boolean;
   end;

   TStatSort = (ssName, ssWeeksPlayed, ssPickPer);

   TPool_Module = class;

   TStatList = class(TList)
   protected
      SortBy      : TStatSort;
      NumGames    : array[1..MAXWEEK] of integer;
      WeekScored  : array[1..MAXWEEK] of boolean;
      procedure   DoOneDrop;
   public
      constructor Create;
      destructor  Destroy; override;
      procedure   ClearItems;
      procedure   AddStat(st: TPlayerStat);
      function    ItemByID(UserID: integer): TPlayerStat;
      function    ItemByIdx(Idx: integer): TPlayerStat;

      procedure   GetNumGames(pm : TPool_Module);
      procedure   CalcPercentages(NumDrop: integer);
      procedure   SetSort(SortParam: TStatSort);
   end;

  TPool_Module = class(TDataModule)
    WABD_PoolSession: TWABD_Session;
    Pool_Main_Form: TWABD_Form;
    UserTable: TTable;
    UserTableUserName: TStringField;
    UserTableUserPass: TStringField;
    UserTableLastLogon: TDateTimeField;
    UserTableNumLogons: TIntegerField;
    GlobalsTable: TTable;
    GlobalsTableCurWeek: TIntegerField;
    TeamsTable: TTable;
    TeamsTableTeamName: TStringField;
    GamesTable: TTable;
    GamesTableWeek: TIntegerField;
    GamesTableGameNum: TIntegerField;
    GamesTableHomeTeamID: TIntegerField;
    GamesTableHomeTeamName: TStringField;
    GamesTableAwayTeamID: TIntegerField;
    GamesTableAwayTeamName: TStringField;
    GamesTableSpread: TFloatField;
    GamesTablePlayed: TBooleanField;
    GamesTableScore1: TIntegerField;
    GamesTableScore2: TIntegerField;
    WeeksTable: TTable;
    WeeksTableWeek: TIntegerField;
    WeeksTableTotGames: TIntegerField;
    WeeksTableLockOutTime: TDateTimeField;
    PicksTable: TTable;
    PicksTableWeek: TIntegerField;
    PicksTablePlayerID: TIntegerField;
    PicksTablePlayerName: TStringField;
    PicksTableGameID: TIntegerField;
    PicksTablePick: TIntegerField;
    PlayerWeeksTable: TTable;
    PlayerWeeksTableWeekID: TIntegerField;
    PlayerWeeksTablePlayerID: TIntegerField;
    PlayerWeeksTablePlayerName: TStringField;
    PlayerWeeksTablePlayedWeek: TBooleanField;
    PlayerWeeksTablePaid: TBooleanField;
    PlayerWeeksTablePaidAmount: TFloatField;
    PlayerWeeksTableComment: TStringField;
    Session1: TSession;
    WABD_Header1: TWABD_Header;
    MainSection1: TWABD_FormSection;
    WABD_Label1: TWABD_Label;
    UserName: TWABD_Edit;
    WABD_Label2: TWABD_Label;
    UserPass: TWABD_Edit;
    LogonBut: TWABD_Button;
    WABD_Label7: TWABD_Label;
    Pool_Bgnd: TWABD_Image;
    NewUserBut: TWABD_Button;
    WABD_Label3: TWABD_Label;
    UserPass2: TWABD_Edit;
    WABD_Label4: TWABD_Label;
    Main_Stat_Label: TWABD_Label;
    AdminForm: TWABD_Form;
    AdminFormHdr: TWABD_Header;
    AdminSection: TWABD_FormSection;
    TableListBox: TWABD_ListBox;
    WABD_Label5: TWABD_Label;
    WABD_Button1: TWABD_Button;
    TableEditForm: TWABD_Form;
    TableEditHeader: TWABD_Header;
    TableEditSection: TWABD_FormSection;
    ReturnToAdminBut: TWABD_Button;
    TableEdit_DataSource: TDataSource;
    TabEdForm_StatLab: TWABD_Label;
    AdminUserOpBut: TWABD_Button;
    GamesTableGameTime: TDateTimeField;
    GamesTableTVStation: TStringField;
    GamesTableGameDay: TStringField;
    WeeksTableLockOutDay: TStringField;
    UsageTable: TTable;
    UsageTableUserID: TIntegerField;
    UsageTableUserName: TStringField;
    UsageTableTime: TDateTimeField;
    UsageTableAction: TIntegerField;
    UsageTableActionDesc: TStringField;
    UserForm: TWABD_Form;
    UserMainHdr: TWABD_Header;
    MainUserSec: TWABD_FormSection;
    LastForm: TWABD_Form;
    GoodbyeHeader: TWABD_Header;
    GlobalsTableEntryCost: TFloatField;
    PicksForm: TWABD_Form;
    PicksHeader: TWABD_Header;
    PicksButSection: TWABD_FormSection;
    SavePicksButton: TWABD_Button;
    CancelPicksButton: TWABD_Button;
    PersonalHdr: TWABD_Header;
    Lockout_Hdr: TWABD_Header;
    PlayerWeeksTableTieBreakerPts: TFloatField;
    UnderConForm: TWABD_Form;
    UnderConHdr: TWABD_Header;
    WABD_FormSection1: TWABD_FormSection;
    ReturnBut: TWABD_Button;
    WABD_FormSection4: TWABD_FormSection;
    WABD_Anchor1: TWABD_Anchor;
    ViewPicksForm: TWABD_Form;
    UserFormStat: TWABD_Header;
    CurTimeHdr: TWABD_Header;
    GoAdminBut: TWABD_Button;
    ViewPicksHdr: TWABD_Header;
    ViewPicksButSec: TWABD_FormSection;
    ViewPicksRefreshBut: TWABD_Button;
    ViewPicksSec: TWABD_FormSection;
    InitFavoritesBut: TWABD_Button;
    TeamsTableTeamShort: TStringField;
    GamesTableHomeShort: TStringField;
    GamesTableAwayShort: TStringField;
    LongNamesCheckbox: TWABD_CheckBox;
    WABD_Button2: TWABD_Button;
    PicksSubHeader: TWABD_Header;
    WABD_Label6: TWABD_Label;
    TieBreakerEdit: TWABD_Edit;
    WABD_Label8: TWABD_Label;
    CommentEdit: TWABD_Edit;
    SpreadBut: TWABD_Button;
    SpreadForm: TWABD_Form;
    SpreadHdr: TWABD_Header;
    SpreadSec: TWABD_FormSection;
    SpreadButSec: TWABD_FormSection;
    SaveSpreadBut: TWABD_Button;
    CancelSpreadBut: TWABD_Button;
    ScoreBut: TWABD_Button;
    ScoreForm: TWABD_Form;
    SpreadSubHdr: TWABD_Header;
    ScoreHdr: TWABD_Header;
    ScoreSec: TWABD_FormSection;
    ScoreButSec: TWABD_FormSection;
    SaveScoreBut: TWABD_Button;
    CancelScoreBut: TWABD_Button;
    WeeksTableScored: TBooleanField;
    UnScoreBut: TWABD_Button;
    UnSubmitBut: TWABD_Button;
    UserTableUserID: TIntegerField;
    TeamsTableTeamID: TIntegerField;
    UsageTableUsageID: TIntegerField;
    MainCancelBut: TWABD_Button;
    ConfirmPassBut: TWABD_Button;
    PreviousWeekForm: TWABD_Form;
    PrevFormHdr: TWABD_Header;
    WABD_FormSection2: TWABD_FormSection;
    PrevWeekCancel: TWABD_Button;
    PrevFormSubHdr: TWABD_Header;
    PrevWeekTable: TWABD_Table;
    PaymentBut: TWABD_Button;
    PayForm: TWABD_Form;
    PayHdr: TWABD_Header;
    PaySec: TWABD_FormSection;
    SeasonPmtCheckbox: TWABD_CheckBox;
    PayEdit: TWABD_Edit;
    WABD_Label9: TWABD_Label;
    WABD_Label10: TWABD_Label;
    PayReturnBut: TWABD_Button;
    AdminLogoffBut: TWABD_Button;
    PayTable: TWABD_Table;
    WeekPaidEdit: TWABD_Edit;
    WABD_Label11: TWABD_Label;
    WABD_Label12: TWABD_Label;
    StatsForm: TWABD_Form;
    Statshdr: TWABD_Header;
    StatsButSec: TWABD_FormSection;
    StatsReturnBut: TWABD_Button;
    StatsSec: TWABD_FormSection;
    SortPerBut: TWABD_Button;
    SortWeekBut: TWABD_Button;
    SortNameBut: TWABD_Button;
    WABD_Label13: TWABD_Label;
    DropWeekEdit: TWABD_Edit;
    StatRefreshBut: TWABD_Button;
    ExpireForm: TWABD_Form;
    ExpireHdr: TWABD_Header;
    ExpireHdr2: TWABD_Header;
    ExpireSec: TWABD_FormSection;
    ExpireAnchor: TWABD_Anchor;
    ExpireLogoffBut: TWABD_Button;
    WABD_Label14: TWABD_Label;
    ScoreWeekEdit: TWABD_Edit;
    ScoreRefreshBut: TWABD_Button;
    ViewSortPicksBut: TWABD_Button;
    ViewSortNameBut: TWABD_Button;
    TeamStatForm: TWABD_Form;
    TeamStatHdr: TWABD_Header;
    TeamStatSec: TWABD_FormSection;
    TeamStatButSec: TWABD_FormSection;
    WABD_Button3: TWABD_Button;
    TeamStatCombo: TWABD_ComboBox;
    TeamSelectBut: TWABD_Button;
    PlayerStatForm: TWABD_Form;
    PlayerStatHdr: TWABD_Header;
    PicksSection: TWABD_FormSection;
    HomeLabel: TWABD_Label;
    AwayLabel: TWABD_Label;
    SpreadLabel: TWABD_Label;
    GameTimeLabel: TWABD_Label;
    GameDayLabel: TWABD_Label;
    TVLabel: TWABD_Label;
    PicksSubHeader2: TWABD_Header;
    LatestNewsAnchor: TWABD_Anchor;
    PicksButton: TWABD_LiveImage;
    ViewPicksButton: TWABD_LiveImage;
    ResultsButton: TWABD_LiveImage;
    StatsButton: TWABD_LiveImage;
    UserLogOffButton: TWABD_LiveImage;
    procedure WABD_Button1Click(Sender: TObject);
    procedure ReturnToAdminButClick(Sender: TObject);
    procedure TableEditFormShow(Sender: TObject);
    procedure AdminUserOpButClick(Sender: TObject);
    procedure GamesTableCalcFields(DataSet: TDataSet);
    procedure WeeksTableCalcFields(DataSet: TDataSet);
    procedure UsageTableCalcFields(DataSet: TDataSet);
    procedure WABD_PoolSessionFirstLogon(Sender: TObject);
    procedure LogonButClick(Sender: TObject);
    procedure UserLogoffButtonClick(Sender: TObject);
    procedure NewUserButClick(Sender: TObject);
    procedure WABD_PoolSessionLogOff(Sender: TObject);
    procedure WABD_PoolSessionTimeOut(Sender: TObject);
    procedure UserFormShow(Sender: TObject);
    procedure PicksButtonClick(Sender: TObject);
    procedure PicksFormShow(Sender: TObject);
    procedure SavePicksButtonClick(Sender: TObject);
    procedure CancelPicksButtonClick(Sender: TObject);
    procedure LastFormShow(Sender: TObject);
    procedure ReturnButClick(Sender: TObject);
    procedure ViewPicksButtonClick(Sender: TObject);
    procedure ResultsButtonClick(Sender: TObject);
    procedure GoAdminButClick(Sender: TObject);
    procedure ViewPicksRefreshButClick(Sender: TObject);
    procedure InitFavoritesButClick(Sender: TObject);
    procedure WABD_Button2Click(Sender: TObject);
    procedure SpreadButClick(Sender: TObject);
    procedure SpreadFormShow(Sender: TObject);
    procedure ScoreButClick(Sender: TObject);
    procedure CancelSpreadButClick(Sender: TObject);
    procedure SaveSpreadButClick(Sender: TObject);
    procedure ScoreFormShow(Sender: TObject);
    procedure CancelScoreButClick(Sender: TObject);
    procedure SaveScoreButClick(Sender: TObject);
    procedure UnScoreButClick(Sender: TObject);
    procedure UnSubmitButClick(Sender: TObject);
    procedure PK_AfterOpen(DataSet: TDataSet);
    procedure PK_NewRecord(DataSet: TDataSet);
    procedure UsageTablePostError(DataSet: TDataSet; E: EDatabaseError;
      var Action: TDataAction);
    procedure MainCancelButClick(Sender: TObject);
    procedure Pool_ModuleCreate(Sender: TObject);
    procedure ConfirmPassButClick(Sender: TObject);
    procedure PrevWeekCancelClick(Sender: TObject);
    procedure PreviousWeekFormShow(Sender: TObject);
    procedure PrevWeekTableClick(Sender: TObject; RowIndex: Integer);
    procedure PaymentButClick(Sender: TObject);
    procedure PayReturnButClick(Sender: TObject);
    procedure PayFormShow(Sender: TObject);
    procedure PayTableClick(Sender: TObject; RowIndex: Integer);
    procedure StatsButtonClick(Sender: TObject);
    procedure StatsReturnButClick(Sender: TObject);
    procedure StatsFormShow(Sender: TObject);
    procedure Pool_ModuleDestroy(Sender: TObject);
    procedure SortPerButClick(Sender: TObject);
    procedure SortWeekButClick(Sender: TObject);
    procedure SortNameButClick(Sender: TObject);
    procedure StatRefreshButClick(Sender: TObject);
    procedure ExpireLogoffButClick(Sender: TObject);
    procedure ViewPicksFormShow(Sender: TObject);
    procedure ScoreRefreshButClick(Sender: TObject);
    procedure ViewSortPicksButClick(Sender: TObject);
    procedure ViewSortNameButClick(Sender: TObject);
    procedure WABD_Button3Click(Sender: TObject);
    procedure TeamStatFormShow(Sender: TObject);
    procedure TeamSelectButClick(Sender: TObject);
  private
    { Private declarations }
    VP_Bold    : Boolean;
    VP_Size    : integer;
    VP_Color   : TColor;
    StatList   : TStatList;
    DidStats   : boolean;
    TeamList   : TStringList;
    procedure  UpdateStatList;
    function   VP_MakeLabel(x,y : integer; const Caption: string): TWABD_Label;
    procedure  SetWinColor(Winner, Pick: integer);
    procedure  GetCurrentWeek;
    function   CheckExpire: boolean;
    procedure  InitButton(li: TWABD_LiveImage; const Caption: string);
    procedure  CreatePickButtons;
  public
    { Public declarations }
    UserID     : integer;
    UserStr    : string;
    CurWeek    : integer;
    CurViewWeek: integer;
    StatTeam   : integer;
    EntryCost  : double;
    LogOnTime  : TDateTime;
    IsAdmin    : boolean;
    IsGuest    : boolean;
    procedure  ForEachTable(EachTable: TEachTable);
    procedure  InitSession;
    procedure  CloseTable(Table: TTable);
    procedure  SetDatabase(Table: TTable);
    procedure  OpenTable(Table: TTable);
    procedure  CreateTable(Table: TTable);
    procedure  DeleteTable(Table: TTable);
    procedure  ClearIndexDefs(Table: TTable);
    procedure  CreateTables;
    procedure  DeleteTables;
    procedure  LogAction(UserID: integer; Action: TLogAction);
    function   CheckPassword: boolean;
    procedure  ViewWeek(ViewWeek: integer; BeforeLockout: boolean; SortType: TViewSortType);
    function   IsBeforeLockout(Week: integer): boolean;
    procedure  TeamClick(Sender: TObject);
    procedure  PlayerClick(Sender: TObject);

    // Payment functions
    function   GetUserPlayed(UserID: integer): boolean;
    function   GetUserOwe(UserID: integer): double;
    function   GetUserOweWeek(UserID, WeekID: integer): double;

    // Scoring Functions
    function   GetRightPicks(UserID, WeekID: integer): integer;
    function   GetPointDif(UserID, WeekID: integer): double;
    function   GetUserScore(UserID, WeekID: integer): double;
    function   GetGameWinner(WeekID, GameID: integer): integer;
    function   GetWeekWinners(WeekID: integer): TStringList;
  end;


var
  Pool_Module: TPool_Module;

implementation

{$R *.DFM}

function DoSortName(Item1, Item2: pointer): integer; forward;
function DoSortWeeks(Item1, Item2: pointer): integer; forward;
function DoSortPer(Item1, Item2: pointer): integer; forward;

// **********************************************************************
// TStatList

constructor TStatList.Create;
begin
   inherited;
end;

destructor TStatList.Destroy;
begin
   ClearItems;
   inherited;
end;

procedure TStatList.ClearItems;
var
   i  : integer;
begin
   for i := Count-1 downto 0 do begin
      TPlayerStat(Items[i]).Free;
   end;
   Clear;
end;

procedure TStatList.AddStat(st: TPlayerStat);
begin
   Add(st);
end;

function TStatList.ItemByIdx(Idx: integer): TPlayerStat;
begin
   Result := TPlayerStat(Items[Idx]);
end;

function TStatList.ItemByID(UserID: integer): TPlayerStat;
var
   i : Integer;
begin
   Result := nil;
   for i := 0 to Count-1 do
      if ItemByIdx(i).UserID = UserID then begin
         Result := ItemByIdx(i);
         exit;
      end;
end;

procedure TStatList.GetNumGames(pm : TPool_Module);
begin
   pm.WeeksTable.First;
   while not pm.WeeksTable.EOF do begin
      NumGames[pm.WeeksTableWeek.AsInteger] := pm.WeeksTableTotGames.AsInteger;
      WeekScored[pm.WeeksTableWeek.AsInteger] := pm.WeeksTableScored.AsBoolean;
      pm.WeeksTable.Next;
   end;
end;

procedure TStatList.DoOneDrop;
var
   i, j     : integer;
   st       : TPlayerStat;
   LowPer   : double;
   Per      : double;
   LowPos   : integer;
begin
   for i := 0 to Count-1 do begin
      st := ItemByIdx(i);
      LowPer := 1e30;
      LowPos := -1;
      for j := 1 to MAXWEEK do begin
         if st.Dropped[j] then continue;
         if not WeekScored[j] then continue;
         if st.Played[j] then begin
            Per := (0.0 + st.NumRight[j]) / NumGames[j];
         end else begin
            Per := 0;
         end;
         if Per < LowPer then begin
            LowPer := Per;
            LowPos := j;
         end;
      end;
      if LowPos<>-1 then begin
         st.Dropped[LowPos] := True;
      end;
   end;
end;

procedure TStatList.CalcPercentages(NumDrop: integer);
var
   i, j     : integer;
   st       : TPlayerStat;
   Tot      : double;
   TotPos   : double;
begin
   { Reset the drops to zero }
   for i := 0 to Count-1 do begin
      st := ItemByIdx(i);
      for j := 1 to MAXWEEK do st.Dropped[j] := False;
   end;

   for i := 1 to NumDrop do
      DoOneDrop;

   for i := 0 to Count-1 do begin
      st := ItemByIdx(i);
      Tot := 0;
      TotPos := 0;
      st.PickPer := 0;
      for j := 1 to MAXWEEK do begin
         if not st.Played[j] then continue;
         if st.Dropped[j] then continue;
         if not WeekScored[j] then continue;
         TotPos := TotPos + NumGames[j];
         Tot    := Tot + st.NumRight[j];
      end;
      if (TotPos > 0) then
         st.PickPer := Tot / TotPos;
   end;
end;

function DoSortName(Item1, Item2: pointer): integer;
begin
   Result := 0;
   if TPlayerStat(Item1).UserName > TPlayerStat(Item2).UserName then Result := 1;
   if TPlayerStat(Item1).UserName < TPlayerStat(Item2).UserName then Result := -1;
end;

function DoSortWeeks(Item1, Item2: pointer): integer;
begin
   Result := 0;
   if TPlayerStat(Item1).WeeksPlayed > TPlayerStat(Item2).WeeksPlayed then Result := -1;
   if TPlayerStat(Item1).WeeksPlayed < TPlayerStat(Item2).WeeksPlayed then Result := 1;
   if Result=0 then
      Result := DoSortName(Item1, Item2);
end;

function DoSortPer(Item1, Item2: pointer): integer;
begin
   Result := 0;
   if TPlayerStat(Item1).PickPer > TPlayerStat(Item2).PickPer then Result := -1;
   if TPlayerStat(Item1).PickPer < TPlayerStat(Item2).PickPer then Result := 1;
   if Result=0 then
      Result := DoSortName(Item1, Item2);
end;

procedure TStatList.SetSort(SortParam: TStatSort);
begin
   case SortParam of
      ssName         : Sort(DoSortName);
      ssWeeksPlayed  : Sort(DoSortWeeks);
      ssPickPer      : Sort(DoSortPer);
   end;
end;

// **********************************************************************
// TPool_Module

procedure TPool_Module.ForEachTable(EachTable: TEachTable);
var
   i : integer;
   c : TComponent;
begin
   for i := 0 to ComponentCount-1 do begin
      c := Components[i];
      if (c is TTable) then
         EachTable(TTable(c));
   end;
end;

procedure TPool_Module.CloseTable(Table: TTable);
begin
   Table.Active := False;
end;

procedure TPool_Module.CreateTable(Table: TTable);
begin
   Table.CreateTable;
end;

procedure TPool_Module.OpenTable(Table: TTable);
begin
   Table.Active := True;
end;

procedure TPool_Module.ClearIndexDefs(Table: TTable);
begin
   Table.IndexDefs.Clear;
end;

procedure TPool_Module.DeleteTable(Table: TTable);
begin
   try
      Table.Active := False;
      Table.DeleteTable;
   except
      on E:Exception do begin end;
   end;
end;

procedure TPool_Module.DeleteTables;
begin
   ForEachTable(DeleteTable);
end;

procedure TPool_Module.CreateTables;
begin
   ForEachTable(CloseTable);
   ForEachTable(ClearIndexDefs);

   with UserTable do begin
      IndexDefs.Add('UserID',   'UserID',   [ixPrimary]);
      IndexDefs.Add('UserName', 'UserName', [ixUnique, ixCaseInsensitive]);
   end;

   with GlobalsTable do begin
   end;

   with TeamsTable do begin
      IndexDefs.Add('TeamID', 'TeamID', [ixPrimary]);
   end;

   with GamesTable do begin
      IndexDefs.Add('GamesIdx', 'WeekID;GameID', [ixPrimary]);
   end;

   with WeeksTable do begin
      IndexDefs.Add('WeekID', 'WeekID', [ixPrimary]);
   end;

   with PicksTable do begin
      IndexDefs.Add('PicksIdx', 'WeekID;PlayerID;GameID', [ixPrimary]);
   end;

   with PlayerWeeksTable do begin
      IndexDefs.Add('PyrWeekIdx', 'WeekID;PlayerID', [ixPrimary]);
   end;

   with UsageTable do begin
      IndexDefs.Add('UsageIdx', 'UsageID', [ixPrimary]);
      IndexDefs.Add('UserIdx', 'UserID;Time', []);
      IndexDefs.Add('TimeIdx', 'Time;UserID', []);
   end;

   ForEachTable(CreateTable);
   ForEachTable(OpenTable);
end;

function TPool_Module.CheckPassword: boolean;
begin
   Result := False;
   UserTable.IndexFieldNames := 'UserName';

   if UpperCase(UserName.Text) = 'GUEST' then begin
      UserID  := GUEST_USER_ID;
      IsGuest := True;
      Result  := True;
      exit;
   end;

   if not UserTable.FindKey([UserName.Text]) then begin
      Main_Stat_Label.Visible := True;
      Main_Stat_Label.Caption := 'Invalid logon';
      exit;
   end;

   if UpperCase(UserPass.Text) = UpperCase(UserTableUserPass.AsString) then begin
      Result := True;
      UserID := UserTableUserID.AsInteger;
      UserStr := UserName.Text;
      LogAction(UserID, Log_Logon);
   end else begin
      Main_Stat_Label.Visible := True;
      Main_Stat_Label.Caption := 'Invalid logon';
   end;
end;

procedure TPool_Module.WABD_Button1Click(Sender: TObject);
var
   t : TTable;
begin
   t := nil;
   case TableListBox.SelIndex of
      0  : t := UserTable;
      1  : t := TeamsTable;
      2  : t := WeeksTable;
      3  : t := PlayerWeeksTable;
      4  : t := GamesTable;
      5  : t := PicksTable;
      6  : t := GlobalsTable;
      7  : t := UsageTable;
   end;

   if t<>nil then begin
      TableEdit_DataSource.DataSet := t;
      TableEditForm.Show;
   end;
end;

procedure TPool_Module.ReturnToAdminButClick(Sender: TObject);
begin
   AdminForm.Show;
end;

procedure TPool_Module.TableEditFormShow(Sender: TObject);
var
   ds : TDataSet;
begin
   ds := TableEdit_DataSource.DataSet;
   ds.Active := True;

   TableEditHeader.Caption := 'Edit Table: ' + ds.Name;
end;

procedure TPool_Module.AdminUserOpButClick(Sender: TObject);
begin
   UserForm.Show;
   GoAdminBut.Visible := True;
end;

procedure TPool_Module.GamesTableCalcFields(DataSet: TDataSet);
begin
   GamesTableGameDay.AsString := FormatDateTime('ddd', GamesTableGameTime.AsDateTime);
end;

procedure TPool_Module.WeeksTableCalcFields(DataSet: TDataSet);
begin
   WeeksTableLockOutDay.AsString := FormatDateTime('ddd', WeeksTableLockOutTime.AsDateTime);
end;

procedure TPool_Module.LogAction(UserID: integer; Action: TLogAction);
begin
   with UsageTable do begin
      Active := True;
      Append;
      UsageTableUserID.AsInteger    := UserID;
      UsageTableTime.AsDateTime     := Now;
      UsageTableAction.AsInteger    := integer(Action);
      try
         Post;
      except
         on Exception do begin end;
      end;
   end;
end;

procedure TPool_Module.UsageTableCalcFields(DataSet: TDataSet);
var
   ac : TLogAction;
   s  : string;
begin
   ac := TLogAction(UsageTableAction.AsInteger);
   case ac of
      Log_CreateTables  : s := 'Created Tables';
      Log_LogOn         : s := 'Log On';
      Log_LogOff        : s := 'Log Off';
      Log_TimeOut       : s := 'Time Out';
      Log_NewUser       : s := 'New User';
      Log_SavePicks     : s := 'Save Picks';
      Log_LookStats     : s := 'Look Stats';
      Log_LookResults   : s := 'Look Results';
      Log_ViewAllPicks  : s := 'View All Picks';
      Log_TeamStats     : s := 'Team Stats';
      Log_PlayerStats   : s := 'Player Stats';
   else
      s := '?';
   end;
   UsageTableActionDesc.AsString := s;
end;

procedure TPool_Module.InitSession;
const
   SesNum : integer = 0;
var
   sl     : TStringList;
begin
   SesNum := SesNum + 1;
   Session1.Active := False;
   Session1.SessionName := Format('Pool_Session_%d', [SesNum]);

   sl := TStringList.Create;
   Session1.GetAliasParams(POOL_ALIAS, sl);
   Session1.NetFileDir  := sl.Values['PATH'];
   sl.Free;
   Session1.Active := True;
end;

procedure TPool_Module.SetDatabase(Table: TTable);
begin
   Table.Active := False;
   Table.SessionName := Session1.SessionName;
   Table.DatabaseName := POOL_ALIAS;
end;

procedure TPool_Module.WABD_PoolSessionFirstLogon(Sender: TObject);
begin
   InitSession;

   ForEachTable(SetDatabase);
   ForEachTable(OpenTable);
   LogOnTime := Now;

   // Find the CurWeek
   GetCurrentWeek;

   // Load the Team List
   TeamsTable.First;
   while not TeamsTable.EOF do begin
      TeamList.AddObject(TeamsTableTeamName.AsString, TObject(TeamsTableTeamID.AsInteger));
      TeamsTable.Next;
   end;
   TeamStatCombo.Lines.Assign(TeamList);

   CreatePickButtons;
end;

procedure TPool_Module.LogonButClick(Sender: TObject);
var
   GoodPass : boolean;
begin
   GoodPass := CheckPassword;

   if not GoodPass then exit;

   // Update Num Logons & Last Logon
   UserTable.IndexFieldNames := 'UserID';
   if UserTable.FindKey([UserID]) then begin
      UserTable.Edit;
      UserTableLastLogon.AsDateTime := Now;
      UserTableNumLogons.AsInteger := UserTableNumLogons.AsInteger + 1;
      UserTable.Post;
   end;

   if UpperCase(UserName.Text) = 'ADMIN' then begin
      IsAdmin := True;
      AdminForm.Show;
   end else begin
      IsAdmin := False;
      UserForm.Show;
   end;
end;

procedure TPool_Module.UserLogoffButtonClick(Sender: TObject);
begin
   WABD_PoolSession.Logoff;
   LastForm.Show;
end;

procedure TPool_Module.NewUserButClick(Sender: TObject);
begin
   Main_Stat_Label.Visible := True;

   UserTable.IndexFieldNames := 'UserName';
   if UserTable.FindKey([UserName.Text]) then begin
      Main_Stat_Label.Caption := 'Sorry, that user name is already being used - try another name.';
      exit;
   end;

   if (UserName.Text='') or (UserPass.Text='') then begin
      Main_Stat_Label.Caption := 'You must enter a name and password';
      exit;
   end;

   WABD_Label3.Visible := True;
   UserPass2.Visible := True;
   LogonBut.Visible := False;
   NewUserBut.Visible := False;

   Main_Stat_Label.Caption := 'Re-enter your password and press "Confirm Password"';
   ConfirmPassBut.Visible := True;
end;

procedure TPool_Module.WABD_PoolSessionLogOff(Sender: TObject);
begin
   if UserID<>GUEST_USER_ID then LogAction(UserID, Log_Logoff);
end;

procedure TPool_Module.WABD_PoolSessionTimeOut(Sender: TObject);
begin
   if UserID<>GUEST_USER_ID then LogAction(UserID, Log_TimeOut);
end;

function Fill(const s: string; Num: integer): string;
begin
   Result := s;
   while Length(Result) < Num do Result := Result + Chr($A0);
end;

procedure TPool_Module.InitButton(li: TWABD_LiveImage; const Caption: string);
var
   x, y  : integer;
   w, h  : integer;
   R     : TRect;
begin
   li.LocalPath   := ExtractFilePath(Application.ExeName);

   with li.Canvas do begin
      Pen.Color    := clBlack;
      Pen.Width    := 2;
      Brush.Style  := bsSolid;
      R := Rect(0, 0, li.ImageWidth, li.ImageHeight);
      Rectangle(R.Left, R.Top, R.Right, R.Bottom);

      Font.Name    := 'Times New Roman';
      Font.Size    := 24;
      Font.Style   := Font.Style + [fsBold];
      Brush.Style  := bsClear;

      w := TextWidth(Caption);
      h := TextHeight(Caption);
      x := li.ImageWidth div 2 - w div 2;
      y := li.ImageHeight div 2 - h div 2;

      Font.Color  := clLtGray;
      TextOut(x+2, y+2, Caption);

      Font.Color  := clBlue;
      TextOut(x-1, y-1, Caption);
   end;
end;

procedure TPool_Module.CreatePickButtons;
begin
   InitButton(PicksButton,       'Make/View Your Picks');
   InitButton(ViewPicksButton,   'View All Players'' Picks');
   InitButton(ResultsButton,     'View Previous Weeks'' Results');
   InitButton(StatsButton,       'View Statistics');
   InitButton(UserLogOffButton,  'Log Off');
end;

procedure TPool_Module.UserFormShow(Sender: TObject);
var
   i        : integer;
   b        : TWABD_Button;
   Played   : boolean;
   Owe      : double;
   s1, s2   : string;
begin
   if IsGuest then begin
      PicksButton.Visible := False;
   end;

   // Check if the user has played, and what they owe
   Played := GetUserPlayed(UserID);
   Owe    := GetUserOwe(UserID);
   if Played then s1 := 'Your picks are entered for this week.'
      else s1 := 'You have not entered any picks this week.';
   if Owe > 0.005 then s2 := Format('You owe $%1.2n.', [Owe])
      else s2 := '';
   if not IsGuest then
      UserFormStat.Caption := s1 + '  ' + s2
   else
      UserFormStat.Caption := '';

   for i := 0 to MainUserSec.ChildCount-1 do begin
      if MainUserSec.Children[i] is TWABD_Button then begin
         b := MainUserSec.Children[i] as TWABD_Button;
         b.Caption := Fill(b.Caption, 30);
      end;
   end;

   UserMainHdr.Caption := Format('Main User Options - Week %d', [CurWeek]);
   CurTimeHdr.Caption := 'Current Time = ' + FormatDateTime('dddd, mmm dd, yyyy, hh:nn ampm', Now);
   if not WeeksTable.FindKey([CurWeek]) then exit;
   LockOut_Hdr.Caption := 'LOCK OUT Time = ' +
      FormatDateTime('dddd, mmm dd, yyyy, hh:nn ampm', WeeksTableLockOutTime.AsDateTime);
end;

procedure TPool_Module.PicksButtonClick(Sender: TObject);
begin
   if IsBeforeLockout(CurWeek) or IsAdmin then begin
      PicksForm.Show;
   end else begin
      UserFormStat.Caption := 'Sorry, you can''t change your picks after the lock-out time.';
   end;
end;

procedure TPool_Module.PicksFormShow(Sender: TObject);
var
   i        : integer;
   wc       : TWABD_ComboBox;
   Row      : integer;
   GameID   : integer;
   Pick     : integer;
   l        : TWABD_Label;
function MakeLab(Col: integer; const Caption: string): TWABD_Label;
begin
   Result := PicksSection.AddControl(TWABD_Label, Col, Row) as TWABD_Label;
   Result.Caption  := Caption;
end;
begin
   PicksHeader.Caption := UserStr + ', Make your Picks!';
   PicksSection.CellPadding := 2;

   // Clear out the old Picks controls first
   for i := PicksSection.ChildCount-1 downto 0 do begin
      if PicksSection.Children[i].Tag<>-1 then
         PicksSection.Children[i].Free;
   end;

   // Create the Picks controls
   if not GamesTable.FindKey([CurWeek]) then exit;

   Row := 1;
   while (GamesTableWeek.AsInteger = CurWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      l := MakeLab(0, GamesTableHomeTeamName.AsString);
      l.Tag := GamesTableHomeTeamID.AsInteger;
      l.CanClick := True;
      l.OnClick := TeamClick;
      l := MakeLab(1, GamesTableAwayTeamName.AsString);
      l.Tag := GamesTableAwayTeamID.AsInteger;
      l.CanClick := True;
      l.OnClick := TeamClick;

      MakeLab(2, Format('%1.1n', [GamesTableSpread.AsFloat]));

      wc := PicksSection.AddControl(TWABD_ComboBox, 3, Row) as TWABD_ComboBox;
      wc.Name := Format('PickSec_Combo_%d', [GameID]);
      wc.Lines.Clear;
      wc.Lines.Add(Fill('No Pick',25));
      wc.Lines.Add(Fill(GamesTableHomeTeamName.AsString,25));
      wc.Lines.Add(Fill(GamesTableAwayTeamName.AsString,25));

      MakeLab(4, FormatDateTime('h:nn ampm', GamesTableGameTime.AsDateTime));
      MakeLab(5, GamesTableGameDay.AsString);
      MakeLab(6, GamesTableTVStation.AsString);

      // Get the User's pick from the database
      if PicksTable.FindKey([CurWeek, UserID, GameID]) then begin
         Pick := PicksTablePick.AsInteger;
      end else begin
         Pick := 0;
      end;
      wc.SelIndex := Pick;

      GamesTable.Next;
      Row := Row + 1;
   end;

   if PlayerWeeksTable.FindKey([CurWeek, UserID]) then begin
      TieBreakerEdit.Text := Format('%1.1n', [PlayerWeeksTableTieBreakerPts.AsFloat]);
      CommentEdit.Text := PlayerWeeksTableComment.AsString;
   end;
end;

procedure TPool_Module.InitFavoritesButClick(Sender: TObject);
var
   ComboName   : string;
   wc          : TWABD_ComboBox;
   GameID      : integer;
   Pick        : integer;
begin
   GamesTable.FindKey([CurWeek]);
   while (GamesTableWeek.AsInteger = CurWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      ComboName := Format('PickSec_Combo_%d', [GameID]);
      wc := PicksSection.ChildByName(ComboName) as TWABD_ComboBox;

      if GamesTableSpread.AsFloat >= 0 then Pick := 1
         else Pick := 2;
      wc.SelIndex := Pick;

      GamesTable.Next;
   end;
   TieBreakerEdit.Text := '42.0';
end;

procedure TPool_Module.SavePicksButtonClick(Sender: TObject);
var
   ComboName   : string;
   wc          : TWABD_ComboBox;
   GameID      : integer;
   Pick        : integer;
   tmp         : double;
begin
   // Save the Picks
   GamesTable.FindKey([CurWeek]);
   while (GamesTableWeek.AsInteger = CurWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      ComboName := Format('PickSec_Combo_%d', [GameID]);
      wc := PicksSection.ChildByName(ComboName) as TWABD_ComboBox;
      Pick := wc.SelIndex;

      // Save the User's Picks to the database
      if PicksTable.FindKey([CurWeek, UserID, GameID]) then begin
         PicksTable.Edit;
      end else begin
         PicksTable.Append;
         PicksTableWeek.AsInteger := CurWeek;
         PicksTablePlayerID.AsInteger := UserID;
         PicksTableGameID.AsInteger := GameID;
      end;
      PicksTablePick.AsInteger := Pick;
      PicksTable.Post;

      GamesTable.Next;
   end;

   // Save the Player Played (as True)
   if PlayerWeeksTable.FindKey([CurWeek, UserID]) then begin
      PlayerWeeksTable.Edit;
   end else begin
      PlayerWeeksTable.Append;
      PlayerWeeksTableWeekID.AsInteger := CurWeek;
      PlayerWeeksTablePlayerID.AsInteger := UserID;
   end;
   PlayerWeeksTablePlayedWeek.AsBoolean := True;

   // Save the Total Points & Comment
   try
      tmp := StrToFloat(TieBreakerEdit.Text);
   except
      tmp := 0;
   end;
   PlayerWeeksTableTieBreakerPts.AsFloat := tmp;
   PlayerWeeksTableComment.AsString := CommentEdit.Text;
   PlayerWeeksTable.Post;

   UserForm.Show;
   LogAction(UserID, Log_SavePicks);
end;

procedure TPool_Module.UnSubmitButClick(Sender: TObject);
begin
   if PlayerWeeksTable.FindKey([CurWeek, UserID]) then begin
      PlayerWeeksTable.Edit;
      PlayerWeeksTablePlayedWeek.AsBoolean := False;
      PlayerWeeksTable.Post;
   end;
   UserForm.Show;
end;

procedure TPool_Module.CancelPicksButtonClick(Sender: TObject);
begin
   UserForm.Show;
end;

procedure TPool_Module.LastFormShow(Sender: TObject);
var
   s : string;
begin
   s := '"Guest"';
   if UserID<>GUEST_USER_ID then
      s := UserStr;

   PersonalHdr.Caption := Format('%s logged off at %s (%s time on)',
      [s, DateTimeToStr(Now),
      FormatDateTime('nn:ss', Now-LogOnTime)]);
end;

procedure TPool_Module.ReturnButClick(Sender: TObject);
begin
   UserForm.Show;
end;

procedure TPool_Module.ViewPicksButtonClick(Sender: TObject);
begin
   if (not IsBeforeLockout(CurWeek)) or IsAdmin then begin
      LogAction(UserID, Log_ViewAllPicks);
      ViewWeek(CurWeek, False, vsName);
      CurViewWeek := CurWeek;
      ViewPicksForm.Show;
   end else begin
      UserFormStat.Caption := 'You can''t view other player''s picks until after the lock-out time.';
   end;
end;

procedure TPool_Module.ResultsButtonClick(Sender: TObject);
begin
   PreviousWeekForm.Show;
end;

procedure TPool_Module.GoAdminButClick(Sender: TObject);
begin
   AdminForm.Show;
end;

procedure TPool_Module.ViewPicksRefreshButClick(Sender: TObject);
begin
   UserForm.Show;
end;

function TPool_Module.VP_MakeLabel(x,y : integer; const Caption: string): TWABD_Label;
begin
   Result := ViewPicksSec.AddControl(TWABD_Label, x, y) as TWABD_Label;
   Result.Caption    := Caption;
   Result.Bold       := VP_Bold;
   Result.FontSize   := VP_Size;
   Result.FontColor  := VP_Color;
end;

procedure TPool_Module.SetWinColor(Winner, Pick: integer);
begin
   VP_Color := clNone;
   VP_Bold := False;
   if Winner=0 then exit;

   if Winner=Pick then begin
      VP_Color := WIN_COLOR;
      VP_Bold  := True;
   end else begin
      VP_Color := LOSE_COLOR;
      VP_Bold  := False;
   end;
end;

function ViewSortName(Item1, Item2: pointer): integer;
var
   v1, v2 : TViewRecord;
begin
   v1 := Item1;
   v2 := Item2;
   Result := AnsiCompareText(v1.UserName, v2.UserName);
end;

function ViewSortPicks(Item1, Item2: pointer): integer;
var
   v1, v2 : TViewRecord;
begin
   v1 := Item1;
   v2 := Item2;
   Result := 0;
   if v1.NumRight < v2.NumRight then Result := 1;
   if v1.NumRight > v2.NumRight then Result := -1;
   if Result = 0 then begin
      if v1.TiePoints < v2.TiePoints then Result := -1;
      if v1.TiePoints > v2.TiePoints then Result := 1;
   end;
end;

procedure TPool_Module.ViewWeek(ViewWeek: integer; BeforeLockout: boolean;
   SortType: TViewSortType);
const
   CLICKSTATS = True;
var
   i           : integer;
   vr          : TViewRecord;
   GameDate    : string;
   ht, at, spd : string;
   PlayerID    : integer;
   GameID      : integer;
   WeekScored  : boolean;
   Winner      : integer;
   Pick        : integer;
   PlayerList  : TList;
   WinnerList  : TStringList;
   // Stats Variables
   GoodPickSum : double;
   NumRight    : integer;
   PickSum     : array[1..15] of double;
   PtsSum      : double;
   PaidSum     : double;
   tmp         : string;
   l           : TWABD_Label;
begin
   // Clear out any old stuff
   for i := ViewPicksSec.ChildCount-1 downto 0 do
      ViewPicksSec.Children[i].Free;

   ViewPicksSec.CellPadding := 2;

   // Initialize the Tables
   GamesTable.FindKey([ViewWeek]);
   GameDate := FormatDateTime('dddd, mmmm dd, yyyy', GamesTableGameTime.AsDateTime);

   // Check if this week was ever scored
   WeekScored := False;
   if WeeksTable.FindKey([ViewWeek]) then
      WeekScored := WeeksTableScored.AsBoolean;   

   // Fill in the Headers
   ViewPicksHdr.Caption := Format('View Picks - Week %d - %s', [ViewWeek, GameDate]);

   VP_Bold := True;
   VP_Size := 4;
   VP_Color := clNone;
   VP_MakeLabel(2,1, '1-Home Team');
   VP_MakeLabel(2,2, 'Win:');
   VP_MakeLabel(2,3, '2-Away Team');

   VP_MakeLabel(0, 6, 'Num');
   VP_MakeLabel(1, 6, 'Name');
   VP_MakeLabel(2, 6, 'Picks');
   for i := 1 to 15 do VP_MakeLabel(3 + (i-1), 6, IntToStr(i));
   VP_MakeLabel(18, 6, 'Pts');
   VP_MakeLabel(19, 6, 'Paid');
   VP_MakeLabel(20, 6, 'Player Comment');
   VP_Bold := False;
   VP_Size := 3;

   // Fill in the Team Names
   GamesTable.FindKey([ViewWeek]);
   i := 0;
   while (GamesTableWeek.AsInteger = ViewWeek) and (not GamesTable.EOF) do begin
      Winner := GetGameWinner(ViewWeek, GamesTableGameNum.AsInteger);

      if LongNamesCheckbox.Checked then begin
         ht := GamesTableHomeTeamName.AsString;
         at := GamesTableAwayTeamName.AsString;
      end else begin
         ht := GamesTableHomeShort.AsString;
         at := GamesTableAwayShort.AsString;
      end;

      spd := Format('%1.1n', [GamesTableSpread.AsFloat]);

      SetWinColor(Winner, 1);
      if WeekScored then VP_MakeLabel(3+i, 0, IntToStr(GamesTableScore1.AsInteger));
      l := VP_MakeLabel(3+i, 1, Format('%s (%s)', [ht, spd]));
      l.CanClick := CLICKSTATS;
      l.Tag := GamesTableHomeTeamID.AsInteger;
      l.OnClick := TeamClick;
      l.Name := 'Team_Label_'+IntToStr(l.Tag);

      SetWinColor(Winner, 2);
      l := VP_MakeLabel(3+i, 3, at);
      l.CanClick := CLICKSTATS;
      l.Tag := GamesTableAwayTeamID.AsInteger;
      l.OnClick := TeamClick;
      l.Name := 'Team_Label_'+IntToStr(l.Tag);
      if WeekScored then VP_MakeLabel(3+i, 4, IntToStr(GamesTableScore2.AsInteger));

      VP_Color := clNone;
      VP_Bold  := False;
      if WeekScored then VP_MakeLabel(3+i, 2, IntToStr(Winner))
         else VP_MakeLabel(3+i, 2, '?');

      GamesTable.Next;
      i := i + 1;
   end;

   // Fill in the Users (who've played) & their picks
   PlayerWeeksTable.FindKey([ViewWeek]);
   PlayerList := TList.Create;
   while (PlayerWeeksTableWeekID.AsInteger = ViewWeek) and (not PlayerWeeksTable.EOF) do begin
      if PlayerWeeksTablePlayedWeek.AsBoolean then begin
         PlayerID := PlayerWeeksTablePlayerID.AsInteger;

         vr := TViewRecord.Create;
         vr.UserID    := PlayerID;
         vr.UserName  := PlayerWeeksTablePlayerName.AsString;
         vr.NumRight  := GetRightPicks(PlayerID, ViewWeek);
         vr.TiePoints := PlayerWeeksTableTieBreakerPts.AsFloat;

         PlayerList.Add(vr);
      end;

      PlayerWeeksTable.Next;
   end;
   if SortType = vsName then
      PlayerList.Sort(ViewSortName)
   else
      PlayerList.Sort(ViewSortPicks);

   GoodPickSum := 0;
   PtsSum      := 0;
   PaidSum     := 0;
   for i := 1 to 15 do PickSum[i] := 0;

   // Now can do it in order...
   for i := 0 to PlayerList.Count-1 do begin
      vr := PlayerList[i];
      PlayerID := vr.UserID;
      tmp := vr.UserName;
      FindRepAll(tmp, Chr(32), Chr(160));
      l := VP_MakeLabel(1, 7 + i, tmp);
      l.CanClick := CLICKSTATS;
      l.Tag := PlayerID;
      l.Name := 'Player_Label_'+IntToStr(PlayerID);
      l.OnClick := PlayerClick;
      VP_MakeLabel(0, 7 + i, IntToStr(i+1));

      if WeekScored then begin
         NumRight := vr.NumRight;
         GoodPickSum := GoodPickSum + NumRight;
         VP_MakeLabel(2, 7 + i, IntToStr(NumRight));
      end;

      // Display the picks
      GamesTable.FindKey([ViewWeek]);
      while (GamesTableWeek.AsInteger = ViewWeek) and (not GamesTable.EOF) do begin
         GameID := GamesTableGameNum.AsInteger;
         Winner := GetGameWinner(ViewWeek, GamesTableGameNum.AsInteger);

         Pick := -1;
         if PicksTable.FindKey([ViewWeek, PlayerID, GameID]) then
            Pick := PicksTablePick.AsInteger;

         if (GameID >= 1) and (GameID <= 15) then
            PickSum[GameID] := PickSum[GameID] + Pick;

         SetWinColor(Winner, Pick);
         VP_MakeLabel(3 + (GameID-1), 7+i, IntToStr(Pick));

         GamesTable.Next;
      end;
      VP_Color := clNone;
      VP_Bold := False;

      if not PlayerWeeksTable.FindKey([ViewWeek, PlayerID]) then continue;
      VP_MakeLabel(18, 7 + i, Format('%1.1n', [PlayerWeeksTableTieBreakerPts.AsFloat]));
      VP_MakeLabel(19, 7 + i, Format('$%1.2n', [PlayerWeeksTablePaidAmount.AsFloat]));
      tmp := PlayerWeeksTableComment.AsString + Chr(160);
      FindRepAll(tmp, Chr(32), Chr(160));
      VP_MakeLabel(20, 7 + i, tmp);

      PtsSum := PtsSum + PlayerWeeksTableTieBreakerPts.AsFloat;
      PaidSum := PaidSum + PlayerWeeksTablePaidAmount.AsFloat;

   end;

   // Show the Averages / Totals
   if PlayerList.Count > 0 then begin
      VP_Bold := True;
      VP_MakeLabel(1, 7 + PlayerList.Count, 'Average');
      VP_MakeLabel(2, 7 + PlayerList.Count, Format('%1.1n', [GoodPickSum / PlayerList.Count]));
      for i := 1 to 15 do begin
         VP_MakeLabel(2 + i, 7 + PlayerList.Count, Format('%1.1n',
            [PickSum[i] / PlayerList.Count]));
      end;
      VP_MakeLabel(2 + 16, 7 + PlayerList.Count, Format('%1.1n', [PtsSum / PlayerList.Count]));
      VP_MakeLabel(2 + 17, 7 + PlayerList.Count, Format('$%1.2n', [PaidSum]));
   end;

   // Display the winners
   if WeekScored then begin
      WinnerList := GetWeekWinners(ViewWeek);
      if WinnerList<>nil then begin
         {for i := 0 to WinnerList.Count-1 do begin
            VP_MakeLabel(1, 8 + PlayerList.Count + i, WinnerList[i]);
         end; }
         VP_Bold := True;
         VP_Color := WIN_COLOR;
         VP_MakeLabel(1, 8 + PlayerList.Count, 'Winner(s): ' + WinnerList.CommaText);
         WinnerList.Free;
      end;
   end;

   PlayerList.Free;
end;

procedure TPool_Module.WABD_Button2Click(Sender: TObject);
begin
   ViewWeek(CurViewWeek, False, vsName);
   ViewPicksForm.Show;
end;

procedure TPool_Module.SpreadButClick(Sender: TObject);
begin
   SpreadForm.Show;
end;

procedure TPool_Module.SpreadFormShow(Sender: TObject);
var
   i        : integer;
   GameID   : integer;
   Row      : integer;
   we       : TWABD_Edit;
   wl       : TWABD_Label;
begin
   for i := SpreadSec.ChildCount-1 downto 0 do
      SpreadSec.Children[i].Free;

   SpreadSec.CellPadding := 2;

   wl := SpreadSec.AddControl(TWABD_Label, 0, 0) as TWABD_Label;
   wl.Caption := 'HOME Team';
   wl.Bold    := True;
   wl := SpreadSec.AddControl(TWABD_Label, 2, 0) as TWABD_Label;
   wl.Caption := 'Spread';
   wl.Bold    := True;
   wl := SpreadSec.AddControl(TWABD_Label, 4, 0) as TWABD_Label;
   wl.Caption := 'AWAY Team';
   wl.Bold    := True;

   Row := 1;
   GamesTable.FindKey([CurWeek]);
   while (GamesTableWeek.AsInteger = CurWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      wl := SpreadSec.AddControl(TWABD_Label, 0, Row) as TWABD_Label;
      wl.Caption := GamesTableHomeTeamName.AsString;

      wl := SpreadSec.AddControl(TWABD_Label, 1, Row) as TWABD_Label;
      wl.Caption := GamesTableHomeShort.AsString;

      we := SpreadSec.AddControl(TWABD_Edit, 2, Row) as TWABD_Edit;
      we.Name := 'SpreadSec_Spread_'+IntToStr(GameID);
      we.Text := Format('%1.1n', [GamesTableSpread.AsFloat]);

      wl := SpreadSec.AddControl(TWABD_Label, 3, Row) as TWABD_Label;
      wl.Caption := GamesTableAwayShort.AsString;

      wl := SpreadSec.AddControl(TWABD_Label, 4, Row) as TWABD_Label;
      wl.Caption := GamesTableAwayTeamName.AsString;

      GamesTable.Next;
      Row := Row + 1;
   end;
end;

procedure TPool_Module.ScoreButClick(Sender: TObject);
begin
   ScoreWeekEdit.Text := IntToStr(CurWeek);
   ScoreForm.Show;
end;

procedure TPool_Module.CancelSpreadButClick(Sender: TObject);
begin
   AdminForm.Show;
end;

function SafeStrToFloat(const s: string): double;
begin
   try
      Result := StrToFloat(s);
   except
      Result := 0;
   end;
end;

function SafeStrToInt(const s: string): integer;
begin
   Result := Round(SafeStrToFloat(s));
end;

procedure TPool_Module.SaveSpreadButClick(Sender: TObject);
var
   GameID : integer;
   we       : TWABD_Edit;
begin
   GamesTable.FindKey([CurWeek]);
   while (GamesTableWeek.AsInteger = CurWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      GamesTable.Edit;
      we := SpreadSec.FindComponent('SpreadSec_Spread_'+IntToStr(GameID)) as TWABD_Edit;
      GamesTableSpread.AsFloat := SafeStrToFloat(we.Text);
      GamesTable.Post;

      GamesTable.Next;
   end;

   AdminForm.Show;
end;

procedure TPool_Module.ScoreFormShow(Sender: TObject);
var
   i        : integer;
   GameID   : integer;
   Row      : integer;
   we       : TWABD_Edit;
   wl       : TWABD_Label;
   ScoreWeek   : integer;
begin
   for i := ScoreSec.ChildCount-1 downto 0 do
      ScoreSec.Children[i].Free;

   ScoreSec.CellPadding := 2;

   ScoreWeek := SafeStrToInt(ScoreWeekEdit.Text);

   Row := 0;
   GamesTable.FindKey([ScoreWeek]);
   while (GamesTableWeek.AsInteger = ScoreWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      wl := ScoreSec.AddControl(TWABD_Label, 0, Row) as TWABD_Label;
      wl.Caption := GamesTableHomeTeamName.AsString;

      wl := ScoreSec.AddControl(TWABD_Label, 1, Row) as TWABD_Label;
      wl.Caption := GamesTableHomeShort.AsString;

      we := ScoreSec.AddControl(TWABD_Edit, 2, Row) as TWABD_Edit;
      we.Name := 'ScoreSec_Home_'+IntToStr(GameID);
      if not GamesTableScore1.IsNull then
         we.Text := Format('%d', [GamesTableScore1.AsInteger])
      else
         we.Text := '';

      we := ScoreSec.AddControl(TWABD_Edit, 3, Row) as TWABD_Edit;
      we.Name := 'ScoreSec_Away_'+IntToStr(GameID);
      if not GamesTableScore2.IsNull then
         we.Text := Format('%d', [GamesTableScore2.AsInteger])
      else
         we.Text := '';

      wl := ScoreSec.AddControl(TWABD_Label, 4, Row) as TWABD_Label;
      wl.Caption := GamesTableAwayShort.AsString;

      wl := ScoreSec.AddControl(TWABD_Label, 5, Row) as TWABD_Label;
      wl.Caption := GamesTableAwayTeamName.AsString;

      GamesTable.Next;
      Row := Row + 1;
   end;
end;

procedure TPool_Module.CancelScoreButClick(Sender: TObject);
begin
   AdminForm.Show;
end;

procedure TPool_Module.SaveScoreButClick(Sender: TObject);
var
   GameID : integer;
   we     : TWABD_Edit;
   rc     : boolean;
   ScoreWeek : integer;
begin
   ScoreWeek := SafeStrToInt(ScoreWeekEdit.Text);

   GamesTable.FindKey([ScoreWeek]);
   while (GamesTableWeek.AsInteger = ScoreWeek) and (not GamesTable.EOF) do begin
      GameID := GamesTableGameNum.AsInteger;

      GamesTable.Edit;
      we := ScoreSec.FindComponent('ScoreSec_Home_'+IntToStr(GameID)) as TWABD_Edit;
      if we.Text<>'' then
         GamesTableScore1.AsInteger := SafeStrToInt(we.Text);
      we := ScoreSec.FindComponent('ScoreSec_Away_'+IntToStr(GameID)) as TWABD_Edit;
      if we.Text<>'' then
         GamesTableScore2.AsInteger := SafeStrToInt(we.Text);
      GamesTable.Post;

      GamesTable.Next;
   end;

   rc := WeeksTable.FindKey([ScoreWeek]);
   Assert(rc);
   WeeksTable.Edit;
   WeeksTableScored.AsBoolean := True;
   WeeksTable.Post;

   AdminForm.Show;
end;

procedure TPool_Module.UnScoreButClick(Sender: TObject);
begin
   if WeeksTable.FindKey([CurWeek]) then begin
      WeeksTable.Edit;
      WeeksTableScored.AsBoolean := False;
      WeeksTable.Post;
   end;

   AdminForm.Show;
end;

procedure TPool_Module.PK_AfterOpen(DataSet: TDataSet);
var
   t : TTable;
begin
   t := DataSet as TTable;
   t.Tag := GetLastPK(t);
end;

procedure TPool_Module.PK_NewRecord(DataSet: TDataSet);
var
   t : TTable;
   f : TField;
begin
   t := DataSet as TTable;
   f := GetRequiredField(t);
   Assert(f<>nil);
   t.Tag := t.Tag + 1;
   f.AsInteger := t.Tag;
end;

procedure TPool_Module.UsageTablePostError(DataSet: TDataSet;
  E: EDatabaseError; var Action: TDataAction);
var
   t : TTable;
   f : TField;
begin
   t := DataSet as TTable;
   Action := daRetry;
   f := GetRequiredField(t);
   f.AsInteger := f.AsInteger + 1;
end;

procedure TPool_Module.MainCancelButClick(Sender: TObject);
begin
   WABD_PoolSession.LogOff;
   LastForm.Show;
end;

procedure TPool_Module.Pool_ModuleCreate(Sender: TObject);
begin
   UserID := GUEST_USER_ID;
   StatList := TStatList.Create;
   TeamList := TStringList.Create;
end;

procedure TPool_Module.Pool_ModuleDestroy(Sender: TObject);
begin
   StatList.Free;
   TeamList.Free;
end;

function TPool_Module.GetUserPlayed(UserID: integer): boolean;
begin
   Result := False;
   if PlayerWeeksTable.FindKey([CurWeek, UserID]) then begin
      Result := PlayerWeeksTablePlayedWeek.AsBoolean;
   end;
end;

function TPool_Module.GetUserOwe(UserID: integer): double;
var
   i : integer;
begin
   Result := 0;
   for i := 1 to CurWeek do begin
      Result := Result + GetUserOweWeek(UserID, i);
   end;
end;

function TPool_Module.GetUserOweWeek(UserID, WeekID: integer): double;
var
   Played : boolean;
   Paid   : boolean;
   rc     : boolean;
   {$IFDEF PAYMENT_PENALTY}
   Early  : boolean;
   {$ENDIF}
begin
   Result := 0;
   if PlayerWeeksTable.FindKey([WeekID, UserID]) then begin
      rc := WeeksTable.FindKey([WeekID]);
      Assert(rc, 'Can''t find week!');
      Played := PlayerWeeksTablePlayedWeek.AsBoolean;
      Paid   := PlayerWeeksTablePaid.AsBoolean;
      if (Played = True) and (Paid = False) then begin
         Result := EntryCost;
         {$IFDEF PAYMENT_PENALTY}
         Result := Result + 0.25;
         Early  := WeeksTableLockOutTime.AsDateTime > Now;
         if not Early then Result := Result + 0.25;
         {$ENDIF}
      end;
   end;
end;

function TPool_Module.GetGameWinner(WeekID, GameID: integer): integer;
var
   s1, s2       : integer;
begin
   Result := 0;
   if GamesTable.FindKey([WeekID, GameID]) then begin
      s1 := GamesTableScore1.AsInteger;
      s2 := GamesTableScore2.AsInteger;
      if s1 > s2 then Result := 1;
      if s2 > s1 then Result := 2;
   end;
end;

function TPool_Module.GetRightPicks(UserID, WeekID: integer): integer;
var
   Winner, Pick : integer;
   GameID       : integer;
   s1, s2       : integer;
begin
   Result := 0;

   if not GamesTable.FindKey([WeekID]) then exit;
   while (not GamesTable.EOF) and (GamesTableWeek.AsInteger = WeekID) do begin
      GameID := GamesTableGameNum.AsInteger;

      // Determine the Game's winner from the score
      s1 := GamesTableScore1.AsInteger;
      s2 := GamesTableScore2.AsInteger;
      Winner := 0;
      if s1 > s2 then Winner := 1;
      if s2 > s1 then Winner := 2;

      // Get the Pick
      Pick := -1;
      if PicksTable.FindKey([WeekID, UserID, GameID]) then
         Pick := PicksTablePick.AsInteger;

      // Inc if Pick = Game's winner
      if Pick = Winner then Result := Result + 1;

      GamesTable.Next;
   end;
end;

function TPool_Module.GetPointDif(UserID, WeekID: integer): double;
var
   PlayerPoints   : double;
   GamePoints     : integer;
   MaxTime        : TDateTime;
begin
   PlayerPoints := 0;
   if PlayerWeeksTable.FindKey([WeekID, UserID]) then
      PlayerPoints := PlayerWeeksTableTieBreakerPts.AsFloat;

   GamePoints := 0;
   MaxTime := 0;
   GamesTable.FindKey([WeekID]);
   while (not GamesTable.EOF) and (GamesTableWeek.AsInteger = WeekID) do begin
      if GamesTableGameTime.AsFloat > MaxTime then begin
         MaxTime := GamesTableGameTime.AsFloat;
         GamePoints := GamesTableScore1.AsInteger + GamesTableScore2.AsInteger;
      end;

      GamesTable.Next;
   end;

   Result := Abs(PlayerPoints - GamePoints);
end;

function TPool_Module.GetUserScore(UserID, WeekID: integer): double;
begin
   Result := GetRightPicks(UserID, WeekID) - GetPointDif(UserID, WeekID) / 100.0;
end;

function SPSort(Item1, Item2: Pointer): Integer;
begin
   Result := 0;
   if TPlayerScorePair(Item1).Score > TPlayerScorePair(Item2).Score then Result := -1;
   if TPlayerScorePair(Item1).Score < TPlayerScorePair(Item2).Score then Result := 1;
end;

function TPool_Module.GetWeekWinners(WeekID: integer): TStringList;
var
   ScoreList   : TList;
   WinList     : TStringList;
   PlayerID    : integer;
   psp         : TPlayerScorePair;
   i           : integer;
   MaxScore    : double;
begin
   Result    := nil;
   ScoreList := TList.Create;

   // Calculate all of the Players scores
   if not PlayerWeeksTable.FindKey([WeekID]) then exit;
   while (not PlayerWeeksTable.EOF) and (PlayerWeeksTableWeekID.AsInteger = WeekID) do begin

      if PlayerWeeksTablePlayedWeek.AsBoolean then begin
         // Add Players score to ScoreList
         PlayerID       := PlayerWeeksTablePlayerID.AsInteger;
         psp            := TPlayerScorePair.Create;
         psp.PlayerID   := PlayerID;
         psp.Score      := GetUserScore(PlayerID, WeekID);
         psp.Name       := PlayerWeeksTablePlayerName.AsString;
         ScoreList.Add(psp);
      end;

      PlayerWeeksTable.Next;
   end;

   // Sort the scores
   ScoreList.Sort(SPSort);

   // Create the WinnersList
   WinList := TStringList.Create;
   MaxScore := TPlayerScorePair(ScoreList[0]).Score;
   for i := 0 to ScoreList.Count-1 do begin
      psp := TPlayerScorePair(ScoreList[i]);
      if psp.Score = MaxScore then begin
         WinList.AddObject(psp.Name, pointer(psp.PlayerID));
      end;
   end;

   // Free the scorelist
   for i := ScoreList.Count-1 downto 0 do
      TObject(ScoreList.Items[i]).Free;
   ScoreList.Free;

   Result := WinList;
end;

procedure TPool_Module.ConfirmPassButClick(Sender: TObject);
begin
   if UserPass.Text <> UserPass2.Text then begin
      Main_Stat_Label.Caption := 'Passwords do not match, try again.';
      WABD_Label3.Visible := False;
      UserPass2.Visible := False;
      ConfirmPassBut.Visible := False;
      LogonBut.Visible := True;
      NewUserBut.Visible := True;
      exit;
   end;

   UserTable.Append;
   UserTableUserName.AsString := UserName.Text;
   UserTableUserPass.AsString := UserPass.Text;
   UserTableLastLogon.AsDateTime := Now;
   UserTableNumLogons.AsInteger := 1;
   UserTable.Post;
   UserID := UserTableUserID.AsInteger;
   LogAction(UserID, Log_NewUser);

   ConfirmPassBut.Visible := False;
   UserForm.Show;
end;

procedure TPool_Module.PrevWeekCancelClick(Sender: TObject);
begin
   UserForm.Show;
end;

function TPool_Module.IsBeforeLockout(Week: integer): boolean;
var
   rc : boolean;
begin
   rc := WeeksTable.FindKey([Week]);
   Assert(rc, Format('Week %d is not in the Weeks Table!', [Week]));
   Result := Now < WeeksTableLockOutTime.AsDateTime;
end;

procedure TPool_Module.PreviousWeekFormShow(Sender: TObject);
var
   ld     : string;
   WeekID : integer;
   Row    : integer;
begin
   PrevFormSubHdr.Caption := '';
   PrevWeekTable.Cells.SafeSetSize(2, 1);

   WeeksTable.First;
   Row := 1;
   while not WeeksTable.EOF do begin
      WeekID := WeeksTableWeek.AsInteger;
      if WeekID > CurWeek then break;
      ld     := FormatDateTime('dddd, mmm dd, yyyy, hh:nn ampm', WeeksTableLockOutTime.AsDateTime);

      Row := Row + 1;
      PrevWeekTable.Cells.SafeSetSize(2, Row);
      PrevWeekTable.Cells[0,Row-1] := IntToStr(WeekID);
      PrevWeekTable.Cells[1,Row-1] := ld;

      WeeksTable.Next;
   end;
end;

procedure TPool_Module.PrevWeekTableClick(Sender: TObject;
  RowIndex: Integer);
var
   Week  : integer;
begin
   Week := RowIndex;

   if (not IsBeforeLockout(Week)) or IsAdmin then begin
      ViewWeek(Week, False, vsName);
      CurViewWeek := Week;
      ViewPicksForm.Show;
   end else begin
      PrevFormSubHdr.Caption := 'Sorry, it is not after that Week''s lockout time yet.';
   end;
end;

procedure TPool_Module.PaymentButClick(Sender: TObject);
begin
   WeekPaidEdit.Text := IntToStr(CurWeek);
   PayForm.Show;
end;

procedure TPool_Module.PayReturnButClick(Sender: TObject);
begin
   AdminForm.Show;
end;

procedure TPool_Module.PayFormShow(Sender: TObject);
var
   UserID   : integer;
   Owe      : double;
   Row      : integer;
begin
   PayEdit.Text := '';
   SeasonPmtCheckbox.Checked := False;

   // Update the Payment table
   PayTable.Cells.SafeSetSize(3, 1);
   
   // Fill Table with users who owe money!
   Row := 1;
   UserTable.First;
   while not UserTable.EOF do begin
      UserID := UserTableUserID.AsInteger;
      Owe := GetUserOwe(UserID);

      if Owe > 0 then begin
         Row := Row + 1;
         PayTable.Cells.SafeSetSize(3, Row);
         PayTable.Cells[0, Row-1] := IntToStr(UserID);
         PayTable.Cells[1, Row-1] := UserTableUserName.AsString;
         PayTable.Cells[2, Row-1] := Format('%1.2n', [Owe]);
      end;

      UserTable.Next;
   end;
end;

procedure TPool_Module.PayTableClick(Sender: TObject; RowIndex: Integer);
var
   UserID, WeekID : integer;
begin
   UserID := StrToInt(PayTable.Cells[0, RowIndex]);
   WeekID := StrToInt(WeekPaidEdit.Text);

   if not SeasonPmtCheckbox.Checked then begin
      // Paid for the Week
      if not PlayerWeeksTable.FindKey([WeekID, UserID]) then
         raise Exception.CreateFmt('PlayerWeek not found!  WK = %d, User = %d', [WeekID, UserID]);
      PlayerWeeksTable.Edit;
      PlayerWeeksTablePaid.AsBoolean := True;
      if PayEdit.Text<>'' then
         PlayerWeeksTablePaidAmount.AsFloat := StrToFloat(PayEdit.Text)
      else
         PlayerWeeksTablePaidAmount.AsFloat := StrToFloat(PayTable.Cells[2, RowIndex]);
      PlayerWeeksTable.Post;
   end else begin
      // Paid for the Season
      WeeksTable.First;
      while not WeeksTable.EOF do begin
         WeekID := WeeksTableWeek.AsInteger;
         if PlayerWeeksTable.FindKey([WeekID, UserID]) then begin
            PlayerWeeksTable.Edit
         end else begin
            PlayerWeeksTable.Append;
            PlayerWeeksTableWeekID.AsInteger := WeekID;
            PlayerWeeksTablePlayerID.AsInteger := UserID;
         end;
         PlayerWeeksTablePaid.AsBoolean := True;
         PlayerWeeksTablePaidAmount.AsFloat := EntryCost;
         PlayerWeeksTable.Post;

         WeeksTable.Next;
      end;
   end;

   PayForm.Show;
end;

procedure TPool_Module.StatsButtonClick(Sender: TObject);
begin
   LogAction(UserID, Log_LookStats);
   StatsForm.Show;
end;

procedure TPool_Module.StatsReturnButClick(Sender: TObject);
begin
   UserForm.Show;
end;


procedure TPool_Module.UpdateStatList;
var
   st       : TPlayerStat;
   WeekID   : integer;
   UserID   : integer;
   NumRgt   : integer;
   i        : integer;
begin
   // loop through each player
   StatList.ClearItems;
   UserTable.First;
   while not UserTable.EOF do begin
      st := TPlayerStat.Create;
      st.UserID := UserTableUserID.AsInteger;
      st.UserName := UserTableUserName.AsString;
      StatList.AddStat(st);

      UserTable.Next;
   end;

   // For each player, enter their Num picks if played
   PlayerWeeksTable.First;
   while not PlayerWeeksTable.EOF do begin
      WeekID := PlayerWeeksTableWeekID.AsInteger;
      UserID := PlayerWeeksTablePlayerID.AsInteger;

      if PlayerWeeksTablePlayedWeek.AsBoolean then begin
         st := StatList.ItemByID(UserID);
         Assert(st<>nil);
         NumRgt := GetRightPicks(UserID, WeekID);
         st.NumRight[WeekID]  := NumRgt;
         st.Played[WeekID]    := True;
         st.WeeksPlayed       := st.WeeksPlayed + 1;
      end;

      PlayerWeeksTable.Next;
   end;

   // Add "Vegas Line" player

   // Remove users who never played
   for i := StatList.Count-1 downto 0 do begin
      st := StatList.ItemByIdx(i);
      if st.WeeksPlayed = 0 then begin
         StatList.Delete(i);
         st.Free;
      end;
   end;
end;

function AddLabel(Sec: TWABD_FormSection_Base; Col, Row: integer; const Caption: string): TWABD_Label;
begin
   Result := Sec.AddControl(TWABD_Label, Col, Row) as TWABD_Label;
   Result.Caption    := Caption;
end;

function AddHeader(Sec: TWABD_FormSection_Base; Col, Row: integer; const Caption: string): TWABD_Label;
begin
   Result := AddLabel(Sec, Col, Row, Caption);
   Result.Bold       := True;
   Result.FontSize   := 4;
end;

procedure TPool_Module.StatsFormShow(Sender: TObject);
var
   i, j : integer;
   st   : TPlayerStat;
   l    : TWABD_Label;
begin
   for i := StatsSec.ChildCount-1 downto 0 do
      StatsSec.Children[i].Free;

   StatsSec.CellPadding := 2;

   // Update the Statistics
   if not DidStats then begin
      // Prepare Stats
      StatList.GetNumGames(Self);
      UpdateStatList;
      StatList.SetSort(ssName);
      DidStats := True;
   end;
   StatList.CalcPercentages(SafeStrToInt(DropWeekEdit.Text));

   // Update the headers
   AddHeader(StatsSec, 0, 1, 'Player Name');
   AddHeader(StatsSec, 1, 1, 'Weeks Played');
   AddHeader(StatsSec, 2, 1, 'Pick %');
   AddHeader(StatsSec, 2, 0, 'Games:');
   for i := 1 to MAXWEEK do begin
      l := AddHeader(StatsSec, 2 + i, 1, IntToStr(i));
      if not StatList.WeekScored[i] then
         l.FontColor := DROP_COLOR;
      AddLabel(StatsSec, 2 + i, 0, IntToStr(StatList.NumGames[i]));
   end;

   // Fill in the Data
   for i := 0 to StatList.Count-1 do begin
      st := StatList.ItemByIdx(i);
      AddLabel(StatsSec, 0, 2 + i, st.UserName);
      AddLabel(StatsSec, 1, 2 + i, Format('%d', [st.WeeksPlayed]));
      AddLabel(StatsSec, 2, 2 + i, Format('%1.1n%%', [st.PickPer * 100]));

      for j := 1 to MAXWEEK do
         if st.Played[j] then begin
            with AddLabel(StatsSec, 2 + j, 2 + i, IntToStr(st.NumRight[j])) do begin
               // Color code dropped scores in red
               if st.Dropped[j] then
                  FontColor := DROP_COLOR;
               if not StatList.WeekScored[j] then
                  FontColor := DROP_COLOR;
            end;
         end;
   end;
end;

procedure TPool_Module.SortPerButClick(Sender: TObject);
begin
   StatList.SetSort(ssPickPer);
   StatsForm.Show;
end;

procedure TPool_Module.SortWeekButClick(Sender: TObject);
begin
   StatList.SetSort(ssWeeksPlayed);
   StatsForm.Show;
end;

procedure TPool_Module.SortNameButClick(Sender: TObject);
begin
   StatList.SetSort(ssName);
   StatsForm.Show;
end;

procedure TPool_Module.StatRefreshButClick(Sender: TObject);
begin
   StatsForm.Show;
end;

procedure TPool_Module.GetCurrentWeek;
var
   OneAfter : boolean;
begin
   GlobalsTable.First;
   CurWeek := GlobalsTableCurWeek.AsInteger;
   EntryCost := GlobalsTableEntryCost.AsFloat;

   WeeksTable.FindKey([CurWeek]);
   if WeeksTableLockOutTime.AsDateTime < Now then begin
      // If past the lockout time, do a check if past the last game
      OneAfter := False;

      GamesTable.FindKey([CurWeek]);
      while (not GamesTable.EOF) and (GamesTableWeek.AsInteger = CurWeek) do begin
         if GamesTableGameTime.AsDateTime > Now then begin
            OneAfter := True;
            Break;
         end;
         GamesTable.Next;
      end;

      if not OneAfter then begin
         CurWeek := CurWeek+1;
         GlobalsTable.Edit;
         GlobalsTableCurWeek.AsInteger := CurWeek;
         GlobalsTable.Post;
      end;
   end;
end;

function TPool_Module.CheckExpire: boolean;
begin
   Result := False;
   {$IFDEF TIME_CHECK}
   if Now > StrToDate('10/25/1997') then begin
      ExpireForm.Show;
      Result := True;
   end;
   {$ENDIF}
end;

procedure TPool_Module.ExpireLogoffButClick(Sender: TObject);
begin
   LastForm.Show;
   WABD_PoolSession.LogOff;
end;

procedure TPool_Module.ViewPicksFormShow(Sender: TObject);
begin
   CheckExpire;
end;

procedure TPool_Module.ScoreRefreshButClick(Sender: TObject);
begin
   ScoreForm.Show;
end;

procedure TPool_Module.TeamClick(Sender: TObject);
begin
   StatTeam := (Sender as TWabd_Label).Tag;
   TeamStatForm.Show;
end;

procedure TPool_Module.PlayerClick(Sender: TObject);
begin
   UnderConForm.Show;
end;


procedure TPool_Module.ViewSortPicksButClick(Sender: TObject);
begin
   ViewWeek(CurViewWeek, False, vsPicks);
end;

procedure TPool_Module.ViewSortNameButClick(Sender: TObject);
begin
   ViewWeek(CurViewWeek, False, vsName);
end;

procedure TPool_Module.WABD_Button3Click(Sender: TObject);
begin
   UserForm.Show;
end;

procedure TPool_Module.TeamStatFormShow(Sender: TObject);
var
   i     : integer;
   Row   : integer;
   l     : TWABD_Label;
begin
   LogAction(UserID, Log_TeamStats);
   TeamsTable.FindKey([StatTeam]);
   TeamStatHdr.Caption := 'Team Stats: ' + TeamsTableTeamName.AsString;

   TeamStatSec.CellPadding := 2;
   for i := TeamStatSec.ChildCount-1 downto 0 do
      TeamStatSec.Children[i].Free;

   AddLabel(TeamStatSec, 0, 0, 'Week');
   AddLabel(TeamStatSec, 1, 0, 'Home Team');
   AddLabel(TeamStatSec, 2, 0, 'Home Score');
   AddLabel(TeamStatSec, 3, 0, 'Spread');
   AddLabel(TeamStatSec, 4, 0, 'Away Team');
   AddLabel(TeamStatSec, 5, 0, 'Away Score');
   AddLabel(TeamStatSec, 6, 0, 'Game Time');
   AddLabel(TeamStatSec, 7, 0, 'TV');

   GamesTable.First;
   Row := 0;
   while not GamesTable.EOF do begin
      if (GamesTableHomeTeamID.AsInteger = StatTeam) or
         (GamesTableAwayTeamID.AsInteger = StatTeam) then begin

      Row := Row + 1;
      AddLabel(TeamStatSec, 0, Row, GamesTableWeek.AsString);

      l := AddLabel(TeamStatSec, 1, Row, GamesTableHomeTeamName.AsString);
      l.Tag := GamesTableHomeTeamID.AsInteger;
      l.CanClick := True;
      l.OnClick := TeamClick;
      l.Name := 'TeamStat_Home_'+IntToStr(Row);
      if GamesTableHomeTeamID.AsInteger = StatTeam then
         l.Bold := True;
      AddLabel(TeamStatSec, 2, Row, GamesTableScore1.AsString + #160);

      AddLabel(TeamStatSec, 3, Row, GamesTableSpread.AsString + #160);

      l := AddLabel(TeamStatSec, 4, Row, GamesTableAwayTeamName.AsString);
      l.Tag := GamesTableAwayTeamID.AsInteger;
      l.CanClick := True;
      l.OnClick := TeamClick;
      l.Name := 'TeamStat_Away_'+IntToStr(Row);
      if GamesTableAwayTeamID.AsInteger = StatTeam then
         l.Bold := True;
      AddLabel(TeamStatSec, 5, Row, GamesTableScore2.AsString + #160);

      AddLabel(TeamStatSec, 6, Row, FormatDateTime('ddd mm/dd/yy hh:nn ampm', GamesTableGameTime.AsDateTime));
      AddLabel(TeamStatSec, 7, Row, GamesTableTVStation.AsString);

      end;

      GamesTable.Next;
   end;
end;

procedure TPool_Module.TeamSelectButClick(Sender: TObject);
var
   CurTeam : string;
begin
   CurTeam := TeamStatCombo.Lines[TeamStatCombo.SelIndex];
   StatTeam := integer(TeamList.Objects[TeamList.IndexOf(CurTeam)]);
   TeamStatForm.Show;
end;


end.
