unit DataThread;

interface

uses
  Classes, SysUtils, Windows, WinSock;

type
   TDataThread = class(TThread)
   protected
      ErrMsg      : string;
      LogMsg      : string;
      LogData     : string;
      LogHdrs     : string;
      DLLName     : string;
      ECB_Result  : string;
      procedure   Execute; override;
      procedure   ProcessRequest(const FileName, Data: string; var R: string);
      procedure   RunDLL(const DLLFileName, Data: string; var Result: string);
      function    FmtHTMLResponse(const HTMLText: string): string;
      procedure   ShowError;
      procedure   UpdateLog;
      procedure   SendFile(const FileName, FileType: string);
   public
      DataSock    : TSocket;
      RemoteHost  : string;
   end;

implementation

uses Main, SockUtil, Isapi, StrUtil;

{ TDataThread }

function TDataThread.FmtHTMLResponse(const HTMLText: string): string;
var
   ResStr : string;
begin
   ResStr := Format(
      'HTTP/1.0 200 OK'+CR+
      'Content-Type: text/html'+CR+
      'Content-Length: %d'+CR+
      'Content:'+CR+CR, [Length(HTMLText)]) + HTMLText;
   Result := ResStr;
end;

procedure Get_HTTP_Response(Socket: TSocket; var FileName, Data, Hdrs: string);
const
   CONLENSTR = 'Content-Length:';
var
   tmp         : string;
   Lines       : TStringList;
   Line1       : string;
   Op          : string;      // Get, Post, etc
   Rem         : string;      // remaining text after the headers
   p, p1       : integer;
begin
   tmp := GetLine(Socket);
   Lines := TSTringList.Create;
   try
      Lines.Text := tmp;

      // Get the FileName
      Line1 := Lines[0];
      p := Pos(' ', Line1);
      Op := UpperCase(Trim(Copy(Line1, 1, p-1)));
      if p<>0 then Line1 := Copy(Line1, p+1, Length(Line1));   // Trim After the first space
      p := Pos(' ', Line1);
      if p<>0 then Line1 := Copy(Line1, 1, p-1);               // Trim Before the next space

      FindRepAll(Line1, '/', '\');
      while Copy(Line1, 1, 1)='\' do
         Line1 := Copy(Line1, 2, Length(Line1));   // Get rid of the first "/"
      FileName := Line1;

      // Check if the filename has data
      Data := '';
      p := Pos('?', FileName);
      if p<>0 then begin
         Data     := Copy(FileName, p+1, Length(FileName));
         FileName := Copy(FileName, 1, p-1);
      end;

      // Get the Headers
      p := Pos(CR+CR, tmp);
      if p=0 then
         p := Length(tmp);
      p1 := Length(Lines[0]) + 3;
      Hdrs := Copy(tmp, p1, p-p1);

      // Get the Data (for a POST)
      if Op = 'POST' then begin
         Rem := Trim(Copy(tmp, p+4, Length(tmp)));

         // Check for a short or long response
         if (Length(Rem) = 0) then begin
            Data := Trim(GetLine(Socket));
            p := Pos(CR+CR, Data);
            if p<>0 then
               Data := Copy(Data, p+4, Length(Data));
         end else begin
            Data := Rem;
         end;
      end;

   finally
      Lines.Free;
   end;
end;

procedure TDataThread.Execute;
var
   ResStr   : string;
   FileName : string;
   Data     : string;
begin
   try
      Get_HTTP_Response(DataSock, FileName, Data, LogHdrs);
      LogMsg := FileName;
      LogData := Data;

      // Perform aliasing
      if UpperCase(FileName) = 'POOL' then FileName := 'WABD_PoolDemo_DLL.DLL';
      if UpperCase(FileName) = '' then FileName := 'default.htm';

      ResStr := '';
      ProcessRequest(FileName, Data, ResStr);

      // Send the Response
      Synchronize(UpdateLog);
   except
      on e: Exception do begin
         ErrMsg := e.Message + ' (Data Thread)';
         Synchronize(ShowError);
         ResStr := FmtHTMLResponse(ErrMsg);
      end;
   end;

   try
      PutLineNoCR(DataSock,  ResStr);
   except
   end;

   closesocket(DataSock);
end;


procedure TDataThread.ShowError;
begin
   MainForm.ErrorLab.Caption := ErrMsg;
   MainForm.Log(MainForm.Hits, RemoteHost, LogMsg, LogData, LogHdrs, ErrMsg);
end;


procedure TDataThread.UpdateLog;
begin
   MainForm.Log(MainForm.Hits, RemoteHost, LogMsg, LogData, LogHdrs, '');
end;

procedure TDataThread.ProcessRequest(const FileName, Data: string; var R: string);
var
   ext   : string;
begin
   ext := UpperCase(ExtractFileExt(FileName));
   
   if ext='.DLL' then begin
      RunDLL(FileName, Data, R);
      exit;
   end;

   if (ext='.HTM') or (ext='.HTML') then begin
      SendFile(FileName, 'text/html');
      exit;
   end;

   if (ext='.JPG') or (ext='.JPEG') then begin
      SendFile(FileName, 'image/jpeg');
      exit;
   end;

   if (ext='.GIF') then begin
      SendFile(FileName, 'image/gif');
      exit;
   end;

   if (ext='.BMP') then begin
      SendFile(FileName, 'image/bmp');
      exit;
   end;

   // This is the default:
   R := '<HTML><BODY>';
   R := R + '<H1>WWW Server</H1><P>';
   R := R + 'Time = ' + FormatDateTime('dddd, mmm dd, yyyy, hh:nn:ss ampm', Now) + ' (EST)<P>';
   R := R + Format('Hits = %1.0n', [MainForm.Hits+0.0]) + '<P>';
   R := R + 'Request = ' + FileName + '  ' + Data;
   R := R + '</BODY></HTML>';
   R := FmtHTMLResponse(R);
end;


function GetServerVariable(hConn: HCONN; VariableName: PChar; Buffer: Pointer;
   var Size: DWORD ): BOOL stdcall;
var
   dt : TDataThread;
begin
   dt := TDataThread(hConn);
   Result := False;

   if UpperCase(VariableName) = 'SCRIPT_NAME' then begin
      StrPCopy(PChar(Buffer), ExtractFileName(dt.DLLName));
      Size := Length(dt.DLLName);
      Result := True;
   end;

   if (UpperCase(VariableName) = 'REMOTE_HOST') or
      (UpperCase(VariableName) = 'REMOTE_ADDR') then begin
      StrPCopy(PChar(Buffer), dt.RemoteHost);
      Size := Length(dt.RemoteHost);
      Result := True;
   end;
end;

function WriteClient(ConnID: HCONN; Buffer: Pointer; var Bytes: DWORD;
   dwReserved: DWORD ): BOOL stdcall;
var
   dt : TDataThread;
begin
   dt := TDataThread(ConnID);
   dt.ECB_Result := dt.ECB_Result + PChar(Buffer);
   Result := True;
end;

function ReadClient(ConnID: HCONN; Buffer: Pointer; var Size: DWORD): BOOL stdcall;
begin
   Result := False;
end;

function ServerSupport(hConn: HCONN; HSERRequest: DWORD; Buffer: Pointer;
   var Size: DWORD; var DataType: DWORD ): BOOL stdcall;
const
   RESPHDR : pchar = 'HTTP/1.0 200 OK'+CR;
var
   Len : integer;
begin
   Result := False;
   if HSERRequest = HSE_REQ_SEND_RESPONSE_HEADER then begin
      Result := True;
      Len := Length(RESPHDR);
      WriteClient(hConn, RESPHDR, Len, 0);
   end;
end;


procedure TDataThread.RunDLL(const DLLFileName, Data: string; var Result: string);
type
   THttpExtensionProc = function(var ECB: TEXTENSION_CONTROL_BLOCK ):DWORD; stdcall;
var
   hi       : THandle;
   fn       : THttpExtensionProc;
   ecb      : TEXTENSION_CONTROL_BLOCK;
begin
   DLLName := DLLFileName; // for the GetServerVariable Function
   Result := '';
   hi := LoadLibrary(PChar(DLLFileName));
   Assert(hi<>0, 'Unable to load DLL: ' + DLLFileName);
   fn := GetProcAddress(hi, 'HttpExtensionProc');
   Assert(Assigned(fn), 'Unable to find HttpExtensionProc in DLL: ' + DLLFileName);

   FillChar(ecb, sizeof(ecb), 0);
   ecb.cbSize              := sizeof(ecb);
   ecb.dwVersion           := HSE_VERSION_MAJOR shl 16 + HSE_VERSION_MINOR;
   ecb.ConnID              := THandle(Self);
   ecb.dwHttpStatusCode    := 200;
   ecb.lpszMethod          := 'POST';
   ecb.lpszQueryString     := PChar(Data);
   ecb.lpszPathInfo        := PChar(DLLFileName);
   ecb.lpszPathTranslated  := PChar(DLLFileName);
   ecb.cbTotalBytes        := 0;

   ecb.GetServerVariable   := GetServerVariable;
   ecb.WriteClient         := WriteClient;
   ecb.ReadClient          := ReadClient;
   ecb.ServerSupportFunction  := ServerSupport;

   ECB_Result := '';
   fn(ecb);
   Result := ECB_Result;
end;

procedure TDataThread.SendFile(const FileName, FileType: string);
var
   ms    : TMemoryStream;
   tmp   : string;
   hdr   : string;
   rc    : integer;
begin
   ms := TMemoryStream.Create;
try
   tmp := FileName;
   FindRepAll(tmp, '/', '\');
   if Copy(tmp, 1, 1)='\' then tmp := Copy(tmp, 2, Length(tmp));  // trim leading "\"
   ms.LoadFromFile(tmp);
   ms.Position := 0;

   // Send the Header
   hdr := Format(
      'HTTP/1.0 200 OK'+CR+
      'Content-Type: %s'+CR+
      'Content-Length: %d'+CR+
      'Content:'+CR, [FileType, ms.Size]);
   PutLine(DataSock, hdr);

   // Send the file
   rc := send(DataSock, ms.Memory^, ms.Size, 0);
   Assert(rc <> SOCKET_ERROR, Format('Unable to Send (rc = %x)', [rc]));

finally
   ms.Free;
end;
end;


end.
